clear
close all

cd D:\MATLAB\work\HrZZ_2013_2015\Nonlinear_MS_experiments

% Nonlin_BSS_MS_data

%%     
dataset = 1;           % 1 - peptide synthesis from June 2013: 9 mixtures,
                       %     25 pure components
                       
                       
kernel = 'gaussian';   % Options: 'polynomial', 'gaussian'; 'exponential'
                       % 'sigmoid', 'laplace', 'multiquad', 'invmultiquad'
                     
fact_type = 0;         % 1 - NMU; 2 - FastNMF (for A) and T_SBL for S (in mapped space)


lin_fact_type = 1;   % for EKM and EFM based nonlinear algorithms
                       % 1 - NMU; 2 - NMF_L0, 3 - FastSepNMF and Smooth L0
                       % 4 - JADE, 5 - SOBI, 6 - EFICA,
                       % 7 - kernel ICA, 8 - TDSEP 

correction = 0;        %  0/1 flag: used to correct bias (DC)term introduced by polynomial, exponential and Gaussian kernels
                       %  0 - no correction; 1 - correction. 

basis_sel = 1;         %  1 - no clustering; 2 - clustering                       
                     
d = 10;                % order of approximate EFM
b = 1;                 % for appoximate polynomial EFM
    
dp = 10;              % degree of poly kernel
bp = 1e0;             % offset of the poly kernel
sig2 = 1.0e0;         % variance of Gaussian kernel, exponential or Laplacian kernels
sig = 1e7;            % standard deviation for sigmoid kernel
c = 1e2;              % offset for sigmoid, multiquadratic and inverse multiquadratic kernels

D = 500;              % subspace dimension for EKM
 
preproc = 1;           % 0 - NO preprocessing; 
                       % 1 - hard thresholding with threshold tau
                       % 2 - soft thresholding with threshold tau
                       % 3 - low-rank decomposition with regularization
                       %     constant lambda
                       % 4 - trimmed threshodling with alpha=3.5
tau = 1e-5;
alpha = 3.5;
lam = 0.05; 

M = 25;              % number of components

K = 5;               % numbr of overlapping sources

if ( strcmp(kernel,'polynomial'))
    par1=dp; par2 = bp;
elseif ( strcmp(kernel,'gaussian'))
    par1=sig2; par2=[];
elseif ( strcmp(kernel,'exponential'))
    par1=sig2; par2=[];
elseif ( strcmp(kernel,'sigmoid'))
    par1 = sig; par2 = c;
elseif ( strcmp(kernel,'laplace'))
    par1 = sig2; par2=[];
elseif ( strcmp(kernel,'multiquad'))
    par1 = c; par2=[];
elseif ( strcmp(kernel,'invmultiquad'))
    par1 = c; par2=[];
end

%% load experimental MS data

[X, S] = load_mass_spectra(dataset);


%%
xm = max(max(X));
X = X/xm;

%% preprocessing of original data (if any...)
if preproc == 1
    Y=(sign(X-tau)+1)/2;
    X=X.*Y;
elseif preproc == 2
    X = max(0,X-tau);
elseif preproc == 3
    X = proximal_gradient_rpca(X, lam);
elseif preproc == 4
    X = max(0,X.*(power(X,alpha)-power(tau,alpha))./power(X,alpha));
end

%% find D-dimensional basis by k-means clustering
if basis_sel == 1
        V = X;
elseif basis_sel == 2
    % eliminate small components
    Xn=sqrt(sum(X.*X));
    [xmax indmax] = max(Xn);
    Xn = Xn./xmax;
    P=Xn-1/100000; P=sign(P); P=(-P+1)/2;
    Xnp=Xn.*P;
    [xd,Ind]=setdiff(Xn,Xnp);
    XR = X;
    
    cnum=1e5;
    if (D <= 20)
         epscnum=9.0;
    else
         epscnum=750.0;
    end
    
    while cnum > epscnum
         [SH,B]=kmeans(XR',D,'distance','sqEuclidean','Replicates',1,'EmptyAction','singleton'); %,'start','sample');
        V = B';
        cnum=cond(V)
    end
    clear SH 
end
  
X = empirical_kernel_map(X,V,kernel,correction,par1,par2);
NSOURCE = size(X,1);
     
%% preprocessing of mapped data (if any...)
if preproc == 1
    Y=(sign(X-tau)+1)/2;
    X=X.*Y;
elseif preproc == 2
    X = max(0,X-tau);
elseif preproc == 4
    X = max(0,X.*(power(X,alpha)-power(tau,alpha))./power(X,alpha));
end

%% Sparseness constrained factorization
% NMU
ts = tic;
SH = linear_factorization(X,lin_fact_type,size(X,1),14);

time_PT_EKM_NMU = toc(ts);

%% annotation with library 
NSOURCE = size(SH,1);
for i=1:NSOURCE
    for j=1:M
        crpcsh(i,j)=SH(i,:)*S(j,:)'/norm(SH(i,:))/norm(S(j,:));
    end
end
 
for j=1:M
    [PT_EKM_NMU_crmax(j) PT_EKM_NMU_crind(j)]=max(crpcsh(:,j));
end

ind=PT_EKM_NMU_crind;
S_PC = SH(ind,:);

display('PT_EKM_NMU_crmax')
PT_EKM_NMU_crmax
display('PT_EKM_NMU_crind')
PT_EKM_NMU_crind
display('PT_EKM_NMU_time')
time_PT_EKM_NMU

