% Perform sparse NMU using user's feedback on the parameter lambda
% [U,V] = sparseNMU(M,L,H,maxiter)
%
% Input.
%   M              : (m x n) matrix to factorize.
%   L              : images have dimension L x H (H = m/L)
%   maxiter        : number of iterations, default = 100.
%
% Output.
%   (U,V) : solution, UV^T "<=" M, U >= 0 sparse, V >= 0 ("."-approximately)

function [U,V] = sparseNMU(M,L,maxiter)

if nargin <= 2
    maxiter = 100;
end
[m,n] = size(M); H = m/L; 
reply = 1;
lambda = input('What is the initial value of lambda in [0,1[? \n'); 
delta = input('What is the lower bound on the number of pixels per cluster in [0,1[? \n'); 
Delta = input('What is the upper bound on the number of pixels per cluster in [delta,1[? \n'); 

k = 1; 
while reply > 0
    [x,y] = recursiveSNMU(M,1,lambda,delta,Delta,maxiter,1); 
    
    close all; affichage(x,1,1,L,H); 
    disp('What do you want to do next?'); 
    disp('0. Stop the recursion'); 
    disp('1. Continue the recursion'); 
    disp('2. Change Sparsity'); 
    reply = input('3. Change bounds on sparsity \n'); 
    
    if reply == 0
        M = max(M-x*y',0); U(:,k) = x; V(:,k) = y; close all; 
        affichage(U,k,4,L,H); 
        return;
    elseif reply == 1
        M = max(M-x*y',0); U(:,k) = x; V(:,k) = y; 
        k = k+1; 
        close all; 
        fprintf('The old value of lambda is %1.2f \n',lambda); 
        lambda = input('What is the new value of lambda? \n'); 
    elseif reply == 2
        fprintf('The old value of lambda is %1.2f \n',lambda); 
        lambda = input('What is the new value for lambda? \n'); 
    elseif reply == 3
        fprintf('The old value of lambda is %1.2f \n',delta); 
        delta = input('What is the new value for delta (lower bound)? \n'); 
        fprintf('The old value of lambda is %1.2f \n',Delta); 
        Delta = input('What is the new value for delta (upper bound)? \n'); 
    end
end